// This is the essential WDSS-II code for determining the hostname.
// If you are having hostname problems, you can use this to troubleshoot
// @author lakshman
// @version $Id: gethostname.cc 15810 2012-01-30 17:12:12Z Valliappa.Lakshmanan $

#include <string>
#include <netdb.h> // gethostbyname
#include <iostream>

namespace {
  void getHostNameAddr (std::string& host, std::string& addr)
  {
    char hostname[PATH_MAX];
    if (!gethostname (hostname, sizeof(hostname))) // maybe fqdn, maybe not
    {
      std::cout << "hostname successful: got " << hostname << "\n";
      struct hostent * hse = ::gethostbyname (hostname);
      if ( hse != 0 )
      {
        std::cout << "network host entry successful. Got: " << hse->h_name << "\n";
        host = hse->h_name; // fqdn

        char* paddr = (hse->h_addr_list)[0];
        if ( paddr != 0 ) {
          for (int j=0; j < hse->h_length; ++j) {
            char tmp[256];
            const int x ( (unsigned char) paddr[j] );
            snprintf (tmp, sizeof(tmp), "%d", x);
            addr += tmp;
            if ( j != (hse->h_length - 1))
              addr += '.';
          }
        }
      }
      else // couldn't get the fqdn; fall back to gethostname()'s result
      {
        std::cout << "network host entry failed. Using gethostname.\n";
        host = hostname;
      }
    } else {
      perror("get hostname failed:");
    }
  }
}

int main(int argc, char* argv[])
{
  std::string host, addr;
  getHostNameAddr(host, addr);
  std::cout << "host=" << host << " addr=" << addr << "\n";
}
